@@ -16,25 +16,19 @@ class CmdMQTTPeers(object):
         """
         Constructor
         """
-        self.__parser = optparse.OptionParser(usage="%prog { -i [-e] | -l [-n HOSTNAME] [-t TOPIC] | "
-                                                    "-s HOSTNAME TAG SHARED_SECRET TOPIC | -d HOSTNAME } [-a] [-v]",
-                                              version="%prog 1.0")
+        self.__parser = optparse.OptionParser(usage="%prog { -p [-e] | -l [-n HOSTNAME] [-t TOPIC] | -m | -s "
+                                                    "HOSTNAME TAG SHARED_SECRET TOPIC | -d HOSTNAME } "
+                                                    "[-a] [-i INDENT] [-v]", version="%prog 1.0")
 
-        # optional...
-        self.__parser.add_option("--import", "-i", action="store_true", dest="import_peers", default=False,
+        # functions...
+        self.__parser.add_option("--import", "-p", action="store_true", dest="import_peers", default=False,
                                  help="import MQTT peers from stdin")
 
-        self.__parser.add_option("--echo", "-e", action="store_true", dest="echo", default=False,
-                                 help="echo stdin to stdout")
-
         self.__parser.add_option("--list", "-l", action="store_true", dest="list", default=False,
                                  help="list the stored MQTT peers to stdout")
 
-        self.__parser.add_option("--hostname", "-n", type="string", nargs=1, action="store", dest="for_hostname",
-                                 help="filter peers with the given hostname substring")
-
-        self.__parser.add_option("--topic", "-t", type="string", nargs=1, action="store", dest="for_topic",
-                                 help="filter peers with the given topic substring")
+        self.__parser.add_option("--missing", "-m", action="store_true", dest="missing", default=False,
+                                 help="list known devices missing from S3 MQTT peers")
 
         self.__parser.add_option("--set", "-s", type="string", nargs=4, action="store", dest="peer",
                                  help="insert or update an MQTT peer")
@@ -42,9 +36,24 @@ class CmdMQTTPeers(object):
         self.__parser.add_option("--delete", "-d", type="string", nargs=1, action="store", dest="delete",
                                  help="delete an MQTT peer")
 
+        # filters...
+        self.__parser.add_option("--hostname", "-n", type="string", nargs=1, action="store", dest="for_hostname",
+                                 help="filter peers with the given hostname substring")
+
+        self.__parser.add_option("--topic", "-t", type="string", nargs=1, action="store", dest="for_topic",
+                                 help="filter peers with the given topic substring")
+
+        # source...
         self.__parser.add_option("--aws", "-a", action="store_true", dest="aws", default=False,
                                  help="Use AWS S3 instead of local storage")
 
+        # output...
+        self.__parser.add_option("--echo", "-e", action="store_true", dest="echo", default=False,
+                                 help="echo stdin to stdout (import only)")
+
+        self.__parser.add_option("--indent", "-i", type="int", nargs=1, action="store", dest="indent",
+                                 help="pretty-print the output with INDENT (not with echo)")
+
         self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                  help="report narrative to stderr")
 
@@ -59,6 +68,9 @@ class CmdMQTTPeers(object):
         if self.__opts.import_peers:
             count += 1
 
+        if self.missing:
+            count += 1
+
         if self.list:
             count += 1
 
@@ -77,6 +89,12 @@ class CmdMQTTPeers(object):
         if self.__opts.list is None and (self.__opts.for_hostname is not None or self.__opts.for_topic is not None):
             return False
 
+        if self.missing and not self.aws:
+            return False
+
+        if self.echo and self.indent is not None:
+            return False
+
         return True
 
 
@@ -96,24 +114,14 @@ class CmdMQTTPeers(object):
 
     # ----------------------------------------------------------------------------------------------------------------
 
-    @property
-    def echo(self):
-        return self.__opts.echo
-
-
     @property
     def list(self):
         return self.__opts.list
 
 
     @property
-    def for_hostname(self):
-        return self.__opts.for_hostname
-
-
-    @property
-    def for_topic(self):
-        return self.__opts.for_topic
+    def missing(self):
+        return self.__opts.missing
 
 
     @property
@@ -141,11 +149,31 @@ class CmdMQTTPeers(object):
         return self.__opts.delete
 
 
+    @property
+    def for_hostname(self):
+        return self.__opts.for_hostname
+
+
+    @property
+    def for_topic(self):
+        return self.__opts.for_topic
+
+
     @property
     def aws(self):
         return self.__opts.aws
 
 
+    @property
+    def echo(self):
+        return self.__opts.echo
+
+
+    @property
+    def indent(self):
+        return self.__opts.indent
+
+
     @property
     def verbose(self):
         return self.__opts.verbose
@@ -156,9 +184,9 @@ class CmdMQTTPeers(object):
     def print_help(self, file):
         self.__parser.print_help(file)
 
-
     def __str__(self, *args, **kwargs):
-        return "CmdMQTTPeers:{import:%s, echo:%s, list:%s, for_hostname:%s, for_topic:%s, set:%s, " \
-               "delete:%s, aws:%s, verbose:%s}" %  \
-               (self.__opts.import_peers, self.echo, self.list, self.for_hostname, self.for_topic, self.__opts.peer,
-                self.__opts.delete, self.aws, self.verbose)
+        return "CmdMQTTPeers:{import:%s, list:%s, for_hostname:%s, for_topic:%s, missing:%s, set:%s, " \
+               "delete:%s, aws:%s, echo:%s, indent:%s, verbose:%s}" %  \
+               (self.__opts.import_peers, self.list, self.for_hostname, self.for_topic, self.missing, self.__opts.peer,
+                self.__opts.delete, self.aws, self.echo, self.indent, self.verbose)
+
